@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -27,6 +27,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 
 #include <glib.h>
 
@@ -57,6 +58,8 @@ static const char *cpin_prefix[] = { "+CPIN:", NULL };
 static const char *clck_prefix[] = { "+CLCK:", NULL };
 static const char *huawei_cpin_prefix[] = { "^CPIN:", NULL };
 static const char *xpincnt_prefix[] = { "+XPINCNT:", NULL };
+static const char *zpinpuk_prefix[] = { "+ZPINPUK:", NULL };
+static const char *oercn_prefix[] = { "_OERCN:", NULL };
 static const char *cpinr_prefixes[] = { "+CPINR:", "+CPINRE:", NULL };
 static const char *epin_prefix[] = { "*EPIN:", NULL };
 static const char *none_prefix[] = { NULL };
@@ -148,8 +151,10 @@ static void at_sim_read_info(struct ofono_sim *sim, int fileid,
 	snprintf(buf, sizeof(buf), "AT+CRSM=192,%i", fileid);
 
 	switch (sd->vendor) {
+	case OFONO_VENDOR_ZTE:
 	case OFONO_VENDOR_HUAWEI:
 	case OFONO_VENDOR_SIERRA:
+	case OFONO_VENDOR_SPEEDUP:
 	case OFONO_VENDOR_QUALCOMM_MSM:
 		strcat(buf, ",0,0,255"); /* Maximum possible length */
 		break;
@@ -539,6 +544,44 @@ error:
 	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
 }
 
+static void zpinpuk_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+	static enum ofono_sim_password_type password_types[] = {
+		OFONO_SIM_PASSWORD_SIM_PIN,
+		OFONO_SIM_PASSWORD_SIM_PUK,
+	};
+
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+ZPINPUK:"))
+		goto error;
+
+	BUILD_PIN_RETRIES_ARRAY(password_types, ARRAY_SIZE(password_types),
+				retries);
+
+	cb(&error, retries, cbd->data);
+
+	return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
+}
+
 static void xpincnt_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -578,6 +621,102 @@ error:
 	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
 }
 
+static void oercn_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+	static enum ofono_sim_password_type password_types[] = {
+		OFONO_SIM_PASSWORD_SIM_PIN,
+		OFONO_SIM_PASSWORD_SIM_PUK,
+	};
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "_OERCN:"))
+		goto error;
+
+	BUILD_PIN_RETRIES_ARRAY(password_types, ARRAY_SIZE(password_types),
+				retries);
+
+	cb(&error, retries, cbd->data);
+
+	return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
+}
+
+static void cpnnum_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	const char *line;
+	int num;
+	char **entries;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	for (num = 0; num < g_at_result_num_response_lines(result); num++)
+		g_at_result_iter_next(&iter, NULL);
+
+	line = g_at_result_iter_raw_line(&iter);
+
+	DBG("%s", line);
+
+	for (i = 0; i < OFONO_SIM_PASSWORD_INVALID; i++)
+		retries[i] = -1;
+
+	entries = g_strsplit(line, "; ", -1);
+
+	for (num = 0; entries[num]; num++) {
+		int retry;
+
+		if (strlen(entries[num]) < 5)
+			continue;
+
+		retry = strtol(entries[num] + 5, NULL, 10);
+		if (retry == 0 && errno == EINVAL)
+			continue;
+
+		if (g_str_has_prefix(entries[num], "PIN1=") == TRUE)
+			retries[OFONO_SIM_PASSWORD_SIM_PIN] = retry;
+		else if (g_str_has_prefix(entries[num], "PUK1=") == TRUE)
+			retries[OFONO_SIM_PASSWORD_SIM_PUK] = retry;
+		else if (g_str_has_prefix(entries[num], "PIN2=") == TRUE)
+			retries[OFONO_SIM_PASSWORD_SIM_PIN2] = retry;
+		else if (g_str_has_prefix(entries[num], "PUK2=") == TRUE)
+			retries[OFONO_SIM_PASSWORD_SIM_PUK2] = retry;
+	}
+
+	g_strfreev(entries);
+
+	cb(&error, retries, cbd->data);
+}
+
 static void at_epin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -675,19 +814,31 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 		if (g_at_chat_send(sd->chat, "AT+XPINCNT", xpincnt_prefix,
 					xpincnt_cb, cbd, g_free) > 0)
 			return;
-
+		break;
+	case OFONO_VENDOR_SPEEDUP:
+		if (g_at_chat_send(sd->chat, "AT+CPNNUM", NULL,
+					cpnnum_cb, cbd, g_free) > 0)
+			return;
+		break;
+	case OFONO_VENDOR_OPTION_HSO:
+		if (g_at_chat_send(sd->chat, "AT_OERCN?", oercn_prefix,
+					oercn_cb, cbd, g_free) > 0)
+			return;
 		break;
 	case OFONO_VENDOR_HUAWEI:
 		if (g_at_chat_send(sd->chat, "AT^CPIN?", huawei_cpin_prefix,
 					huawei_cpin_cb, cbd, g_free) > 0)
 			return;
-
+		break;
+	case OFONO_VENDOR_ZTE:
+		if (g_at_chat_send(sd->chat, "AT+ZPINPUK=?", zpinpuk_prefix,
+					zpinpuk_cb, cbd, g_free) > 0)
+			return;
 		break;
 	case OFONO_VENDOR_MBM:
 		if (g_at_chat_send(sd->chat, "AT*EPIN?", epin_prefix,
 					at_epin_cb, cbd, g_free) > 0)
 			return;
-
 		break;
 	default:
 		if (g_at_chat_send(sd->chat, "AT+CPINR", cpinr_prefixes,
@@ -1118,12 +1269,28 @@ static struct ofono_sim_driver driver = {
 	.query_locked		= at_pin_query_enabled,
 };
 
+static struct ofono_sim_driver driver_noef = {
+	.name			= "atmodem-noef",
+	.probe			= at_sim_probe,
+	.remove			= at_sim_remove,
+	.read_imsi		= at_read_imsi,
+	.query_passwd_state	= at_pin_query,
+	.query_pin_retries	= at_pin_retries_query,
+	.send_passwd		= at_pin_send,
+	.reset_passwd		= at_pin_send_puk,
+	.lock			= at_pin_enable,
+	.change_passwd		= at_change_passwd,
+	.query_locked		= at_pin_query_enabled,
+};
+
 void at_sim_init(void)
 {
 	ofono_sim_driver_register(&driver);
+	ofono_sim_driver_register(&driver_noef);
 }
 
 void at_sim_exit(void)
 {
 	ofono_sim_driver_unregister(&driver);
+	ofono_sim_driver_unregister(&driver_noef);
 }
