@@ -41,6 +41,8 @@ struct modem_info {
 	char *syspath;
 	char *devname;
 	char *driver;
+	char *vendor;
+	char *model;
 	GSList *devices;
 	struct ofono_modem *modem;
 	const char *sysattr;
@@ -521,13 +523,66 @@ static gboolean setup_telit(struct modem_info *modem)
 	return TRUE;
 }
 
-static gboolean setup_zte(struct modem_info *modem)
+static gboolean setup_simcom(struct modem_info *modem)
 {
-	const char *aux = NULL, *mdm = NULL, *qcdm = NULL;
+	const char *mdm = NULL, *aux = NULL, *gps = NULL, *diag = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
 
+	for (list = modem->devices; list; list = list->next) {
+		struct device_info *info = list->data;
+
+		DBG("%s %s %s %s", info->devnode, info->interface,
+						info->number, info->label);
+
+		if (g_strcmp0(info->label, "aux") == 0) {
+			aux = info->devnode;
+			if (mdm != NULL)
+				break;
+		} else if (g_strcmp0(info->label, "modem") == 0) {
+			mdm = info->devnode;
+			if (aux != NULL)
+				break;
+		} else if (g_strcmp0(info->interface, "255/255/255") == 0) {
+			if (g_strcmp0(info->number, "00") == 0)
+				diag = info->devnode;
+			else if (g_strcmp0(info->number, "01") == 0)
+				gps = info->devnode;
+			else if (g_strcmp0(info->number, "02") == 0)
+				aux = info->devnode;
+			else if (g_strcmp0(info->number, "03") == 0)
+				mdm = info->devnode;
+		}
+	}
+
+	if (aux == NULL || mdm == NULL)
+		return FALSE;
+
+	DBG("modem=%s aux=%s gps=%s diag=%s", mdm, aux, gps, diag);
+
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "Data", aux);
+	ofono_modem_set_string(modem->modem, "GPS", gps);
+
+	return TRUE;
+}
+
+static gboolean setup_zte(struct modem_info *modem)
+{
+	const char *aux = NULL, *mdm = NULL, *qcdm = NULL;
+	const char *modem_intf;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	if (g_strcmp0(modem->model, "0016") == 0 ||
+				g_strcmp0(modem->model, "0017") == 0 ||
+				g_strcmp0(modem->model, "0117") == 0)
+		modem_intf = "02";
+	else
+		modem_intf = "03";
+
 	for (list = modem->devices; list; list = list->next) {
 		struct device_info *info = list->data;
 
@@ -547,9 +602,7 @@ static gboolean setup_zte(struct modem_info *modem)
 				qcdm = info->devnode;
 			else if (g_strcmp0(info->number, "01") == 0)
 				aux = info->devnode;
-			else if (g_strcmp0(info->number, "02") == 0)
-				mdm = info->devnode;
-			else if (g_strcmp0(info->number, "03") == 0)
+			else if (g_strcmp0(info->number, modem_intf) == 0)
 				mdm = info->devnode;
 		}
 	}
@@ -606,7 +659,6 @@ static struct {
 	{ "gobi",	setup_gobi,	},
 	{ "sierra",	setup_sierra	},
 	{ "huawei",	setup_huawei	},
-	{ "huaweicdma",	setup_huawei	},
 	{ "speedupcdma",setup_speedup	},
 	{ "speedup",	setup_speedup	},
 	{ "linktop",	setup_linktop	},
@@ -614,6 +666,7 @@ static struct {
 	{ "novatel",	setup_novatel	},
 	{ "nokia",	setup_nokia	},
 	{ "telit",	setup_telit	},
+	{ "simcom",	setup_simcom	},
 	{ "zte",	setup_zte	},
 	{ "samsung",	setup_samsung	},
 	{ }
@@ -663,6 +716,8 @@ static void destroy_modem(gpointer data)
 	g_free(modem->syspath);
 	g_free(modem->devname);
 	g_free(modem->driver);
+	g_free(modem->vendor);
+	g_free(modem->model);
 	g_free(modem);
 }
 
@@ -705,7 +760,8 @@ static gint compare_device(gconstpointer a, gconstpointer b)
 }
 
 static void add_device(const char *syspath, const char *devname,
-			const char *driver, struct udev_device *device)
+			const char *driver, const char *vendor,
+			const char *model, struct udev_device *device)
 {
 	struct udev_device *intf;
 	const char *devpath, *devnode, *interface, *number, *label, *sysattr;
@@ -737,6 +793,8 @@ static void add_device(const char *syspath, const char *devname,
 		modem->syspath = g_strdup(syspath);
 		modem->devname = g_strdup(devname);
 		modem->driver = g_strdup(driver);
+		modem->vendor = g_strdup(vendor);
+		modem->model = g_strdup(model);
 
 		modem->sysattr = get_sysattr(driver);
 
@@ -802,6 +860,7 @@ static struct {
 	{ "alcatel",	"option",	"1bbb", "0017"	},
 	{ "novatel",	"option",	"1410"		},
 	{ "zte",	"option",	"19d2"		},
+	{ "simcom",	"option",	"05c6", "9000"	},
 	{ "telit",	"usbserial",	"1bc7"		},
 	{ "telit",	"option",	"1bc7"		},
 	{ "nokia",	"option",	"0421", "060e"	},
@@ -814,7 +873,8 @@ static struct {
 static void check_usb_device(struct udev_device *device)
 {
 	struct udev_device *usb_device;
-	const char *driver, *syspath, *devname;
+	const char *syspath, *devname, *driver;
+	const char *vendor = NULL, *model = NULL;
 
 	usb_device = udev_device_get_parent_with_subsystem_devtype(device,
 							"usb", "usb_device");
@@ -854,18 +914,25 @@ static void check_usb_device(struct udev_device *device)
 
 			if (vendor_list[i].vid == NULL) {
 				driver = vendor_list[i].driver;
+				vendor = vid;
+				model = pid;
 				break;
 			}
 
 			if (g_str_equal(vendor_list[i].vid, vid) == TRUE) {
 				if (vendor_list[i].pid == NULL) {
-					if (driver == NULL)
+					if (driver == NULL) {
 						driver = vendor_list[i].driver;
+						vendor = vid;
+						model = pid;
+					}
 					continue;
 				}
 
 				if (g_strcmp0(vendor_list[i].pid, pid) == 0) {
 					driver = vendor_list[i].driver;
+					vendor = vid;
+					model = pid;
 					break;
 				}
 			}
@@ -875,7 +942,7 @@ static void check_usb_device(struct udev_device *device)
 			return;
 	}
 
-	add_device(syspath, devname, driver, device);
+	add_device(syspath, devname, driver, vendor, model, device);
 }
 
 static void check_device(struct udev_device *device)
